@@ -435,6 +435,7 @@ bool reread_config(void)
 
 	(void)report_config_errors(&err_type, NULL, config_errs_to_log);
 	config_Free(config_struct);
+	LogDebug(COMPONENT_CONFIG, "Config reread successfully");
 	return true;
 }
 
@@ -832,6 +833,7 @@ int init_server_pkgs(void)
 	LogEvent(COMPONENT_INIT, "ID Mapper successfully initialized.");
 
 	connection_manager__init();
+	LogEvent(COMPONENT_INIT, "Connection Manager initialized.");
 	return 0;
 }
 
@@ -1006,6 +1008,7 @@ static void nfs_Init(const nfs_start_info_t *p_start_info)
 #endif
 
 #ifdef USE_MONITORING
+	/* initializing nfs ganesha metrics */
 	nfs_metrics__init();
 #endif
 
