@@ -38,6 +38,7 @@
 #include <pthread.h>
 #include <signal.h> /* for sigaction */
 #include <errno.h>
+#include <dlfcn.h>
 #include "fsal.h"
 #include "log.h"
 #include "gsh_rpc.h"
@@ -431,7 +432,7 @@ int main(int argc, char *argv[])
 
 	/* Make sure Linux file i/o will return with error
 	 * if file size is exceeded. */
-#ifdef _LINUX
+#ifdef linux
 	signal(SIGXFSZ, SIG_IGN);
 #endif
 
